/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is released under the GPL with the additional exemption that
 * compiling, linking, and/or using OpenSSL is allowed.
 */

#ifndef GINFO_H
#define GINFO_H

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <dlfcn.h>
#include <gtk/gtk.h>

#include "ginfo_plugin.h"

#define AUTHOR "Andrea Marchesini <bakunin@autistici.org>"
#define COPYRIGHT "copyright GPL 2004"

typedef struct _doc_
{
  char *text;

  GtkWidget *(*func) (void);

  struct _doc_ *next;

} doc;

typedef struct _module_
{
  void *handle;

  char *file;
  char *name;
  int genre;
  char **words;

  doc *docs;
  doc *docs_last;

  struct _module_ *next;
} module;

typedef struct _tooltip_
{

  module *module;
  char *tooltip;

  GtkWidget *(*func) (void);

  struct _tooltip_ *next;

} tooltip;

extern module *module_head;
extern module *module_as_default;
extern tooltip *tooltip_head;
extern int tooltip_time;
extern int tooltip_change_time;
extern int tooltip_done;
extern int tooltip_request;
extern GtkWidget *mainwin;
extern char *graphic_color;
extern GtkWidget *graphic_frame;
extern int nobootsplash;

extern GtkWidget *tutor_win;

/* modules.c */
int modules_readdir (void);
int modules_check (char *);
void modules_close (void);
void modules_graphic_init (void);
void modules_graphic_text (char *, ...);
void modules_graphic_close (void);

/* main.c */
void fatal (char *, ...);
void msg (char *, ...);
void usage (int);
char *trim (char *);

/* graphic.c */
void graphic_init (void);

/* tooltips.c */
void tooltips_add (ginfo_tips *, int, module *);
void tooltips_timer (void);
int tooltips_timeout (gpointer);
void tooltips_close (void);

/* config.c */
void config_read (void);
void config_write (void);

/* tutor.c */
void tutor_init (void);
void tutor_container (GtkWidget *(*)(void));
GtkWidget *tutor_container_default (void);

/* menu.c */
void menu_preference (GtkWidget *, gpointer);
void menu_modules (GtkWidget *, gpointer);
void menu_about (GtkWidget *, gpointer);
void menu_quit (GtkWidget *, gpointer);
void menu_nt (GtkWidget *, gpointer);

/* search.c */
void search_char (char *);

/* docs.c */
int docs_add (ginfo_docs *, int, module *);

#endif
