/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is released under the GPL with the additional exemption that
 * compiling, linking, and/or using OpenSSL is allowed.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"
#include "ginfo_logo.xpm"

module *module_head = NULL;
module *module_as_default = NULL;
GtkWidget *modules_window = NULL;
GtkWidget *modules_label = NULL;

int
modules_readdir (void)
{
  DIR *d;
  struct dirent *dir;
  struct stat st;
  char *a;

  if (!(d = opendir (GINFO_PATH_MODULES)))
    return 1;

  if (!nobootsplash)
    {
      modules_graphic_init ();

      while (gtk_events_pending ())
	gtk_main_iteration ();

      usleep (500000);
    }

  while ((dir = readdir (d)) != NULL)
    {
      int n;

      if (strcmp (dir->d_name + (strlen (dir->d_name) - 3), ".so"))
	continue;

      n = strlen (dir->d_name) + strlen (GINFO_PATH_MODULES) + 2;
      if (!(a = (char *) malloc (n * sizeof (char))))
	fatal ("Error: memory.");

      snprintf (a, n, "%s/%s", GINFO_PATH_MODULES, dir->d_name);

      if (!lstat (a, &st) && !S_ISDIR (st.st_mode))
	{
	  if (!modules_check (a))
	    {
	      if (!nobootsplash)
		modules_graphic_text ("Check module %s", a);
	    }
	  else if (!nobootsplash)
	    modules_graphic_text ("Ignore file %s", a);

	  if (!nobootsplash)
	    {
	      while (gtk_events_pending ())
		gtk_main_iteration ();
	    }
	}

      free (a);
    }

  if (!nobootsplash)
    {
      modules_graphic_close ();

      while (gtk_events_pending ())
	gtk_main_iteration ();
    }

  closedir (d);

  return 0;
}

int
modules_check (char *a)
{
  void *lib;
  int (*func0) (void);
  char *(*func1) (void);
  int (*func2) (void);
  char **(*func3) (void);
  ginfo_tips *(*func4) (int *);
  ginfo_docs *(*func5) (int *);
  int d = 0;
  char *name;
  int genre;
  char **words;
  ginfo_tips *tooltips;
  ginfo_docs *docs;
  int n;
  module *m;

  if (!(lib = dlopen (a, RTLD_LAZY))) {
    msg(dlerror());
    return 1;
    }

  if ((func0 = dlsym (lib, "module_default")))
    {
      d = func0 ();
    }

  if (!(func1 = dlsym (lib, "module_name")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(name = func1 ()))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func2 = dlsym (lib, "module_genre")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(genre = func2 ()))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func3 = dlsym (lib, "module_words")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(words = func3 ()))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func5 = dlsym (lib, "module_docs")) || !(docs = func5 (&n)))
    {
      dlclose (lib);
      return 1;
    }

  if (!(m = (module *) malloc (sizeof (module))))
    fatal ("Error: memory.");

  m->docs = NULL;

  if (docs_add (docs, n, m))
    {
      dlclose (lib);
      free (m);
      return 1;
    }

  if (!(m->file = strdup (a)))
    fatal ("Error: memory");

  m->handle = lib;
  m->name = name;
  m->genre = genre;
  m->words = words;

  m->next = module_head;
  module_head = m;

  if (d)
    {
      if (module_as_default)
	{
	  msg ("Error: more one module_as_default in this session: %s\n"
	       "My module_as_default is: %s\n", a, module_as_default->file);
	}
      else
	{
	  module_as_default = m;
	}
    }

  if ((func4 = dlsym (lib, "module_tooltips")) != NULL
      && (tooltips = func4 (&n)) != NULL)
    tooltips_add (tooltips, n, m);

  return 0;
}

void
modules_graphic_init (void)
{

  GtkWidget *vbox;
  GdkPixmap *pixmap;
  GdkPixmap *mask;
  GtkWidget *image;

  modules_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (modules_window), PACKAGE);
  gtk_window_set_position (GTK_WINDOW (modules_window), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (modules_window), FALSE);
  gtk_widget_set_size_request (modules_window, 200, 200);
  gtk_widget_realize (modules_window);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (modules_window), vbox);

  pixmap = gdk_pixmap_create_from_xpm_d (modules_window->window, &mask, NULL,
					 ginfo_logo);
  image = gtk_image_new_from_pixmap (pixmap, mask);
  gtk_widget_show (image);

  gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 0);

  modules_label = gtk_label_new ("Test module:");
  gtk_widget_show (modules_label);
  gtk_box_pack_start (GTK_BOX (vbox), modules_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (modules_label), GTK_JUSTIFY_CENTER);

  modules_label = gtk_label_new ("initialize data");
  gtk_widget_show (modules_label);
  gtk_box_pack_start (GTK_BOX (vbox), modules_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (modules_label), GTK_JUSTIFY_CENTER);

  gtk_widget_show (modules_window);
}

void
modules_graphic_text (char *str, ...)
{

  char buf[100];
  va_list va;
  va_start (va, str);

  vsnprintf (buf, 100, str, va);
  gtk_label_set_text (GTK_LABEL (modules_label), buf);

  va_end (va);
}

void
modules_graphic_close (void)
{
  gtk_widget_destroy (modules_window);
}


void
modules_close (void)
{
  module *t;

  t = module_head;

  while (t)
    {
      dlclose (t->handle);

      module_head = t->next;
      free (t);

      t = module_head;
    }
}
